@@ -1,8 +1,7 @@
+# pylint: disable=invalid-name
 import warnings
 from datetime import datetime, timezone
-from typing import Dict, Callable, Any, Optional
-import copy
-from fcache.cache import FileCache
+from typing import Callable, Optional
 from apscheduler.job import Job
 from apscheduler.schedulers.background import BackgroundScheduler
 from apscheduler.triggers.interval import IntervalTrigger
@@ -13,10 +12,29 @@ from UnleashClient.strategies import ApplicationHostname, Default, GradualRollou
 from UnleashClient.constants import METRIC_LAST_SENT_TIME, DISABLED_VARIATION, ETAG
 from .utils import LOGGER
 from .deprecation_warnings import strategy_v2xx_deprecation_check
+from .cache import BaseCache, FileCache
 
 # pylint: disable=dangerous-default-value
 class UnleashClient:
-    """Client implementation."""
+    """
+    A client for the Unleash feature toggle system.
+
+    :param url: URL of the unleash server, required.
+    :param app_name: Name of the application using the unleash client, required.
+    :param  environment: Name of the environment using the unleash client, optional & defaults to "default".
+    :param instance_id: Unique identifier for unleash client instance, optional & defaults to "unleash-client-python"
+    :param refresh_interval: Provisioning refresh interval in seconds, optional & defaults to 15 seconds
+    :param refresh_jitter: Provisioning refresh interval jitter in seconds, optional & defaults to None
+    :param metrics_interval: Metrics refresh interval in seconds, optional & defaults to 60 seconds
+    :param metrics_jitter: Metrics refresh interval jitter in seconds, optional & defaults to None
+    :param disable_metrics: Disables sending metrics to unleash server, optional & defaults to false.
+    :param custom_headers: Default headers to send to unleash server, optional & defaults to empty.
+    :param custom_options: Default requests parameters, optional & defaults to empty.  Can be used to skip SSL verification.
+    :param custom_strategies: Dictionary of custom strategy names : custom strategy objects.
+    :param cache_directory: Location of the cache directory. When unset, FCache will determine the location.
+    :param verbose_log_level: Numerical log level (https://docs.python.org/3/library/logging.html#logging-levels) for cases where checking a feature flag fails.
+    :param cache: Custom cache implementation that extends UnleashClient.cache.BaseCache.  When unset, UnleashClient will use Fcache.
+    """
     def __init__(self,
                  url: str,
                  app_name: str,
@@ -31,27 +49,10 @@ class UnleashClient:
                  custom_headers: Optional[dict] = None,
                  custom_options: Optional[dict] = None,
                  custom_strategies: Optional[dict] = None,
-                 cache_directory: str = None,
+                 cache_directory: Optional[str] = None,
                  project_name: str = None,
-                 verbose_log_level: int = 30) -> None:
-        """
-        A client for the Unleash feature toggle system.
-
-        :param url: URL of the unleash server, required.
-        :param app_name: Name of the application using the unleash client, required.
-        :paramgit  environment: Name of the environment using the unleash client, optional & defaults to "default".
-        :param instance_id: Unique identifier for unleash client instance, optional & defaults to "unleash-client-python"
-        :param refresh_interval: Provisioning refresh interval in ms, optional & defaults to 15 seconds
-        :param refresh_jitter: Provisioning refresh interval jitter in s, optional & defaults to None
-        :param metrics_interval: Metrics refresh interval in ms, optional & defaults to 60 seconds
-        :param metrics_jitter: Metrics refresh interval jitter in s, optional & defaults to None
-        :param disable_metrics: Disables sending metrics to unleash server, optional & defaults to false.
-        :param custom_headers: Default headers to send to unleash server, optional & defaults to empty.
-        :param custom_options: Default requests parameters, optional & defaults to empty.
-        :param custom_strategies: Dictionary of custom strategy names : custom strategy objects
-        :param cache_directory: Location of the cache directory. When unset, FCache will determine the location
-        :param verbose_log_level: Numerical log level (https://docs.python.org/3/library/logging.html#logging-levels) for cases where checking a feature flag fails.
-        """
+                 verbose_log_level: int = 30,
+                 cache: Optional[BaseCache] = None) -> None:
         custom_headers = custom_headers or {}
         custom_options = custom_options or {}
         custom_strategies = custom_strategies or {}
@@ -77,14 +78,16 @@ class UnleashClient:
         self.unleash_verbose_log_level = verbose_log_level
 
         # Class objects
-        self.cache = FileCache(self.unleash_instance_id, app_cache_dir=cache_directory)
         self.features: dict = {}
         self.scheduler = BackgroundScheduler()
         self.fl_job: Job = None
         self.metric_job: Job = None
-        self.cache[METRIC_LAST_SENT_TIME] = datetime.now(timezone.utc)
-        self.cache[ETAG] = ''
-        self.cache.sync()
+
+        self.cache = cache or FileCache(self.unleash_app_name, directory=cache_directory)
+        self.cache.mset({
+            METRIC_LAST_SENT_TIME: datetime.now(timezone.utc),
+            ETAG: ''
+        })
 
         # Mappings
         default_strategy_mapping = {
@@ -111,11 +114,21 @@ class UnleashClient:
         Initializes client and starts communication with central unleash server(s).
 
         This kicks off:
+
         * Client registration
         * Provisioning poll
         * Stats poll
 
-        :return:
+        This will raise an exception on registration if the URL is invalid. It is done automatically if called inside a context manager as in:
+
+        .. code-block:: python
+
+            with UnleashClient(
+                url="https://foo.bar",
+                app_name="myClient1",
+                instance_id="myinstanceid"
+                ) as client:
+                pass
         """
         # Only perform initialization steps if client is not initialized.
         if not self.is_initialized:
@@ -140,7 +153,7 @@ class UnleashClient:
                     "custom_headers": self.unleash_custom_headers,
                     "custom_options": self.unleash_custom_options,
                     "features": self.features,
-                    "ondisk_cache": self.cache
+                    "cache": self.cache
                 }
 
                 # Register app
@@ -149,7 +162,7 @@ class UnleashClient:
                                     self.unleash_metrics_interval, self.unleash_custom_headers,
                                     self.unleash_custom_options, self.strategy_mapping)
 
-                fetch_and_load_features(**fl_args)
+                fetch_and_load_features(**fl_args)  # type: ignore
 
                 # Start periodic jobs
                 self.scheduler.start()
@@ -177,19 +190,17 @@ class UnleashClient:
         else:
             warnings.warn("Attempted to initialize an Unleash Client instance that has already been initialized.")
 
-    def destroy(self):
+    def destroy(self) -> None:
         """
         Gracefully shuts down the Unleash client by stopping jobs, stopping the scheduler, and deleting the cache.
 
         You shouldn't need this too much!
-
-        :return:
         """
         self.fl_job.remove()
         if self.metric_job:
             self.metric_job.remove()
         self.scheduler.shutdown()
-        self.cache.delete()
+        self.cache.destroy()
 
     @staticmethod
     def _get_fallback_value(fallback_function: Callable, feature_name: str, context: dict) -> bool:
@@ -209,15 +220,18 @@ class UnleashClient:
         Checks if a feature toggle is enabled.
 
         Notes:
+
         * If client hasn't been initialized yet or an error occurs, flat will default to false.
 
         :param feature_name: Name of the feature
         :param context: Dictionary with context (e.g. IPs, email) for feature toggle.
         :param default_value: Allows override of default value. (DEPRECIATED, used fallback_function instead!)
         :param fallback_function: Allows users to provide a custom function to set default value.
-        :return: True/False
+        :return: Feature flag result
         """
         context = context or {}
+
+        # Update context with static values
         context.update(self.unleash_static_context)
 
         if self.is_initialized:
@@ -240,11 +254,12 @@ class UnleashClient:
         Checks if a feature toggle is enabled.  If so, return variant.
 
         Notes:
+
         * If client hasn't been initialized yet or an error occurs, flat will default to false.
 
         :param feature_name: Name of the feature
         :param context: Dictionary with context (e.g. IPs, email) for feature toggle.
-        :return: Dict with variant and feature flag status.
+        :return: Variant and feature flag status.
         """
         context = context or {}
         context.update(self.unleash_static_context)
