@@ -1,3 +1,4 @@
+from typing import Tuple
 import requests
 from UnleashClient.constants import REQUEST_TIMEOUT, FEATURES_URL
 from UnleashClient.utils import LOGGER, log_resp_info
@@ -9,7 +10,8 @@ def get_feature_toggles(url: str,
                         instance_id: str,
                         custom_headers: dict,
                         custom_options: dict,
-                        project: str = None) -> dict:
+                        project: str = None,
+                        cached_etag: str = '') -> Tuple[dict, str]:
     """
     Retrieves feature flags from unleash central server.
 
@@ -23,7 +25,8 @@ def get_feature_toggles(url: str,
     :param custom_headers:
     :param custom_options:
     :param project:
-    :return: Feature flags if successful, empty dict if not.
+    :param cached_etag:
+    :return: (Feature flags, etag) if successful, ({},'') if not
     """
     try:
         LOGGER.info("Getting feature flag.")
@@ -33,6 +36,9 @@ def get_feature_toggles(url: str,
             "UNLEASH-INSTANCEID": instance_id
         }
 
+        if cached_etag:
+            headers['If-None-Match'] = cached_etag
+
         base_url = f"{url}{FEATURES_URL}"
         base_params = {}
 
@@ -44,13 +50,20 @@ def get_feature_toggles(url: str,
                             params=base_params,
                             timeout=REQUEST_TIMEOUT, **custom_options)
 
-        if resp.status_code != 200:
+        if resp.status_code not in [200, 304]:
             log_resp_info(resp)
             LOGGER.warning("Unleash Client feature fetch failed due to unexpected HTTP status code.")
             raise Exception("Unleash Client feature fetch failed!")
 
-        return resp.json()
+        etag = ''
+        if 'etag' in resp.headers.keys():
+            etag = resp.headers['etag']
+
+        if resp.status_code == 304:
+            return None, etag
+
+        return resp.json(), etag
     except Exception as exc:
         LOGGER.exception("Unleash Client feature fetch failed due to exception: %s", exc)
 
-    return {}
+    return {}, ''
