@@ -1,14 +1,18 @@
 import time
 import json
+import warnings
+
 import pytest
 import responses
 from UnleashClient import UnleashClient
 from UnleashClient.strategies import Strategy
-from tests.utilities.testing_constants import URL, ENVIRONMENT, APP_NAME, INSTANCE_ID, REFRESH_INTERVAL, \
-    METRICS_INTERVAL, DISABLE_METRICS, DISABLE_REGISTRATION, CUSTOM_HEADERS, CUSTOM_OPTIONS, PROJECT_NAME, PROJECT_URL
+from tests.utilities.testing_constants import URL, ENVIRONMENT, APP_NAME, INSTANCE_ID, REFRESH_INTERVAL, REFRESH_JITTER, \
+    METRICS_INTERVAL, METRICS_JITTER, DISABLE_METRICS, DISABLE_REGISTRATION, CUSTOM_HEADERS, CUSTOM_OPTIONS, PROJECT_NAME, PROJECT_URL, \
+    ETAG_VALUE
 from tests.utilities.mocks.mock_features import MOCK_FEATURE_RESPONSE, MOCK_FEATURE_RESPONSE_PROJECT
 from tests.utilities.mocks.mock_all_features import MOCK_ALL_FEATURES
 from UnleashClient.constants import REGISTER_URL, FEATURES_URL, METRICS_URL
+from UnleashClient.cache import FileCache
 
 
 class EnvironmentStrategy(Strategy):
@@ -29,27 +33,32 @@ class EnvironmentStrategy(Strategy):
         return default_value
 
 
+@pytest.fixture
+def cache(tmpdir):
+    return FileCache(APP_NAME, directory=tmpdir.dirname)
+
+
 @pytest.fixture()
-def unleash_client(tmpdir):
+def unleash_client(cache):
     unleash_client = UnleashClient(
         URL,
         APP_NAME,
         refresh_interval=REFRESH_INTERVAL,
         metrics_interval=METRICS_INTERVAL,
-        cache_directory=tmpdir.dirname
+        cache=cache
     )
     yield unleash_client
     unleash_client.destroy()
 
 
 @pytest.fixture()
-def unleash_client_project(tmpdir):
+def unleash_client_project(cache):
     unleash_client = UnleashClient(
         URL,
         APP_NAME,
         refresh_interval=REFRESH_INTERVAL,
         metrics_interval=METRICS_INTERVAL,
-        cache_directory=tmpdir.dirname,
+        cache=cache,
         project_name=PROJECT_NAME
     )
     yield unleash_client
@@ -57,19 +66,19 @@ def unleash_client_project(tmpdir):
 
 
 @pytest.fixture()
-def unleash_client_nodestroy(tmpdir):
+def unleash_client_nodestroy(cache):
     unleash_client = UnleashClient(
         URL,
         APP_NAME,
         refresh_interval=REFRESH_INTERVAL,
         metrics_interval=METRICS_INTERVAL,
-        cache_directory=tmpdir.dirname
+        cache=cache
     )
     yield unleash_client
 
 
 @pytest.fixture()
-def unleash_client_toggle_only(tmpdir):
+def unleash_client_toggle_only(cache):
     unleash_client = UnleashClient(
         URL,
         APP_NAME,
@@ -77,7 +86,7 @@ def unleash_client_toggle_only(tmpdir):
         metrics_interval=METRICS_INTERVAL,
         disable_registration=True,
         disable_metrics=True,
-        cache_directory=str(tmpdir)
+        cache=cache
     )
     yield unleash_client
     unleash_client.destroy()
@@ -96,14 +105,18 @@ def test_UC_initialize_full():
                            ENVIRONMENT,
                            INSTANCE_ID,
                            REFRESH_INTERVAL,
+                           REFRESH_JITTER,
                            METRICS_INTERVAL,
+                           METRICS_JITTER,
                            DISABLE_METRICS,
                            DISABLE_REGISTRATION,
                            CUSTOM_HEADERS,
                            CUSTOM_OPTIONS)
     assert client.unleash_instance_id == INSTANCE_ID
     assert client.unleash_refresh_interval == REFRESH_INTERVAL
+    assert client.unleash_refresh_jitter == REFRESH_JITTER
     assert client.unleash_metrics_interval == METRICS_INTERVAL
+    assert client.unleash_metrics_jitter == METRICS_JITTER
     assert client.unleash_disable_metrics == DISABLE_METRICS
     assert client.unleash_disable_registration == DISABLE_REGISTRATION
     assert client.unleash_custom_headers == CUSTOM_HEADERS
@@ -121,7 +134,7 @@ def test_UC_type_violation():
 def test_uc_lifecycle(unleash_client):
     # Set up API
     responses.add(responses.POST, URL + REGISTER_URL, json={}, status=202)
-    responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_FEATURE_RESPONSE, status=200)
+    responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_FEATURE_RESPONSE, status=200, headers={'etag': ETAG_VALUE})
     responses.add(responses.POST, URL + METRICS_URL, json={}, status=202)
 
     # Create Unleash client and check initial load
@@ -130,8 +143,12 @@ def test_uc_lifecycle(unleash_client):
     assert unleash_client.is_initialized
     assert len(unleash_client.features) >= 4
 
+    # Simulate caching
+    responses.add(responses.GET, URL + FEATURES_URL, json={}, status=304, headers={'etag': ETAG_VALUE})
+    time.sleep(16)
+
     # Simulate server provisioning change
-    responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_ALL_FEATURES, status=200)
+    responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_ALL_FEATURES, status=200, headers={'etag': 'W/somethingelse'})
     time.sleep(30)
     assert len(unleash_client.features) >= 9
 
@@ -252,6 +269,16 @@ def test_uc_is_enabled_error_states(unleash_client):
     assert unleash_client.is_enabled("ThisFlagDoesn'tExist", fallback_function=lambda x, y: True)
 
 
+@responses.activate
+def test_uc_context_manager(unleash_client_nodestroy):
+    responses.add(responses.POST, URL + REGISTER_URL, json={}, status=202)
+    responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_FEATURE_RESPONSE, status=200)
+    responses.add(responses.POST, URL + METRICS_URL, json={}, status=202)
+
+    with unleash_client_nodestroy as unleash_client:
+        assert unleash_client.is_initialized
+
+
 @responses.activate
 def test_uc_not_initialized_isenabled():
     unleash_client = UnleashClient(URL, APP_NAME)
@@ -362,3 +389,38 @@ def test_uc_server_error_recovery(unleash_client):
     responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_FEATURE_RESPONSE, status=200)
     time.sleep(20)
     assert unleash_client.is_enabled("testFlag")
+
+
+def test_uc_with_invalid_url():
+    unleash_client = UnleashClient("thisisnotavalidurl", APP_NAME)
+
+    with pytest.raises(ValueError):
+        unleash_client.initialize_client()
+
+
+def test_uc_with_network_error():
+    unleash_client = UnleashClient("https://thisisavalidurl.com", APP_NAME)
+    unleash_client.initialize_client()
+
+    assert unleash_client.is_enabled
+
+
+@responses.activate
+def test_uc_multiple_initializations(unleash_client):
+    # Set up API
+    responses.add(responses.POST, URL + REGISTER_URL, json={}, status=202)
+    responses.add(responses.GET, URL + FEATURES_URL, json=MOCK_FEATURE_RESPONSE, status=200, headers={'etag': ETAG_VALUE})
+    responses.add(responses.POST, URL + METRICS_URL, json={}, status=202)
+
+    # Create Unleash client and check initial load
+    unleash_client.initialize_client()
+    time.sleep(1)
+    assert unleash_client.is_initialized
+    assert len(unleash_client.features) >= 4
+
+    with warnings.catch_warnings(record=True) as w:
+        # Try and initialize client again.
+        unleash_client.initialize_client()
+
+    assert len(w) == 1
+    assert "initialize" in str(w[0].message)
