@@ -6,7 +6,6 @@ from UnleashClient.variants import Variants
 from UnleashClient.constants import FEATURES_URL, FAILED_STRATEGIES
 from tests.utilities.mocks import MOCK_ALL_FEATURES
 from tests.utilities.testing_constants import DEFAULT_STRATEGY_MAPPING
-from tests.utilities.decorators import cache_full, cache_custom  # noqa: F401
 
 
 def test_loader_initialization(cache_full):  # noqa: F811
@@ -48,13 +47,12 @@ def test_loader_refresh_strategies(cache_full):  # noqa: F811
     # Simulate update mutation
     mock_updated = copy.deepcopy(MOCK_ALL_FEATURES)
     mock_updated["features"][4]["strategies"][0]["parameters"]["percentage"] = 60
-    temp_cache[FEATURES_URL] = mock_updated
-    temp_cache.sync()
+    temp_cache.set(FEATURES_URL, mock_updated)
 
     load_features(temp_cache, in_memory_features, DEFAULT_STRATEGY_MAPPING)
 
     assert in_memory_features["GradualRolloutUserID"].strategies[0].parameters["percentage"] == 60
-    assert len(temp_cache[FAILED_STRATEGIES]) == 1
+    assert len(temp_cache.get(FAILED_STRATEGIES)) == 1
 
 
 def test_loader_refresh_variants(cache_full):  # noqa: F811
@@ -67,8 +65,7 @@ def test_loader_refresh_variants(cache_full):  # noqa: F811
     # Simulate update mutation
     mock_updated = copy.deepcopy(MOCK_ALL_FEATURES)
     mock_updated["features"][8]["variants"][0]["name"] = "VariantA"
-    temp_cache[FEATURES_URL] = mock_updated
-    temp_cache.sync()
+    temp_cache.set(FEATURES_URL, mock_updated)
 
     load_features(temp_cache, in_memory_features, DEFAULT_STRATEGY_MAPPING)
 
