@@ -7,7 +7,9 @@ APP_NAME = "pytest"
 ENVIRONMENT = "unit"
 INSTANCE_ID = "123"
 REFRESH_INTERVAL = 15
+REFRESH_JITTER = None
 METRICS_INTERVAL = 10
+METRICS_JITTER = None
 DISABLE_METRICS = True
 DISABLE_REGISTRATION = True
 CUSTOM_HEADERS = {"name": "My random header."}
@@ -21,6 +23,7 @@ PROJECT_URL = f"{URL}{FEATURES_URL}?project=ivan"
 # Constants
 IP_LIST = "69.208.0.0/29,70.208.1.1,2001:db8:1234::/48,2002:db8:1234:0000:0000:0000:0000:0001"
 PROJECT_NAME = 'ivan'
+ETAG_VALUE = 'W/"730-v0ozrE11zfZK13j7rQ5PxkXfjYQ"'
 
 # Mapping
 DEFAULT_STRATEGY_MAPPING = {
